@@ -7,7 +7,10 @@ use {
         environment::{canonicalize_path, Environment, RustEnvironment},
         project_layout::initialize_project,
         py_packaging::{
-            binary::{EmbeddedPythonContext, LibpythonLinkMode, PythonBinaryBuilder},
+            binary::{
+                EmbeddedPythonContext, LibpythonLinkMode, PythonBinaryBuilder,
+                DEFAULT_PYTHON_CONFIG_FILENAME,
+            },
             distribution::AppleSdkInfo,
         },
         starlark::eval::{EvaluationContext, EvaluationContextBuilder},
@@ -18,7 +21,6 @@ use {
     starlark_dialect_build_targets::ResolvedTarget,
     std::{
         collections::HashMap,
-        convert::TryInto,
         fs::create_dir_all,
         io::{BufRead, BufReader},
         path::{Path, PathBuf},
@@ -67,7 +69,7 @@ pub fn find_pyoxidizer_config_file_env(logger: &slog::Logger, start_dir: &Path)
 
     if let Ok(path) = std::env::var("OUT_DIR") {
         warn!(logger, "looking for config file in ancestry of {}", path);
-        let res = find_pyoxidizer_config_file(&Path::new(&path));
+        let res = find_pyoxidizer_config_file(Path::new(&path));
         if res.is_some() {
             return res;
         }
@@ -96,9 +98,8 @@ impl BuildEnvironment {
         logger: &slog::Logger,
         target_triple: &str,
         artifacts_path: &Path,
-        target_python_path: &Path,
+        pyo3_config_path: impl AsRef<Path>,
         libpython_link_mode: LibpythonLinkMode,
-        libpython_filename: Option<&Path>,
         apple_sdk_info: Option<&AppleSdkInfo>,
     ) -> Result<Self> {
         let rust_environment = env
@@ -116,36 +117,13 @@ impl BuildEnvironment {
         // Tells any invoked pyoxidizer process to reuse artifacts if they are up to date.
         envs.insert("PYOXIDIZER_REUSE_ARTIFACTS".to_string(), "1".to_string());
 
-        // Set PYTHON_SYS_EXECUTABLE so python3-sys uses our distribution's Python to configure
-        // itself.
-        // TODO the build environment requiring use of target arch executable prevents
-        // cross-compiling. We should be able to pass in all state without having to
-        // run an executable in a build script.
+        // Give PyO3 an explicit configuration file to use. This bypasses the dynamic interpreter
+        // probing that PyO3's build script normally performs.
         envs.insert(
-            "PYTHON_SYS_EXECUTABLE".to_string(),
-            target_python_path.display().to_string(),
+            "PYO3_CONFIG_FILE".to_string(),
+            pyo3_config_path.as_ref().display().to_string(),
         );
 
-        let mut rust_flags = vec![];
-
-        // If linking against an existing dynamic library on Windows, add the path to that
-        // library so the linker can find it.
-        if let Some(libpython_filename) = libpython_filename {
-            if target_triple.contains("-windows-") {
-                let libpython_dir = libpython_filename
-                    .parent()
-                    .ok_or_else(|| anyhow!("unable to find parent directory of python DLL"))?;
-
-                rust_flags.push(format!("-L{}", libpython_dir.display()));
-            }
-        }
-
-        // static-nobundle link kind requires nightly Rust compiler until
-        // https://github.com/rust-lang/rust/issues/37403 is resolved.
-        if target_triple.contains("-windows-") {
-            envs.insert("RUSTC_BOOTSTRAP".to_string(), "1".to_string());
-        }
-
         // When targeting Apple platforms and using Apple SDKs, you can very
         // easily run into SDK and toolchain compatibility issues when your
         // local SDK or toolchain is older than the one used to produce the
@@ -172,9 +150,11 @@ impl BuildEnvironment {
 
             let deployment_target_name = sdk.supported_targets.get(&sdk_info.platform).ok_or_else(|| {
                 anyhow!("could not find settings for target {} (this shouldn't happen)", &sdk_info.platform)
-            })?.deployment_target_setting_name.as_ref().ok_or_else(|| {
-                anyhow!("unable to identify deployment target environment variable for {} (please report this bug)", &sdk_info.platform)
-            })?;
+            })?.deployment_target_setting_name.clone().unwrap_or_else(|| {
+                warn!(logger, "Apple SDK does not define deployment target name; assuming MACOSX_DEPLOYMENT_TARGET");
+                warn!(logger, "(If you see this message, the SDK you are attempting to use may be too old and build failures may occur.)");
+                "MACOSX_DEPLOYMENT_TARGET".to_string()
+            });
 
             // SDKROOT will instruct rustc and potentially other tools to use exactly this SDK.
             envs.insert("SDKROOT".to_string(), sdk.path.display().to_string());
@@ -182,14 +162,13 @@ impl BuildEnvironment {
             // This (e.g. MACOSX_DEPLOYMENT_TARGET) will instruct compilers to target a specific
             // minimum version of the target platform. We respect an explicit value if one
             // is given.
-            if envs.get(deployment_target_name).is_none() {
-                envs.insert(
-                    deployment_target_name.to_string(),
-                    sdk_info.deployment_target.clone(),
-                );
+            if envs.get(&deployment_target_name).is_none() {
+                envs.insert(deployment_target_name, sdk_info.deployment_target.clone());
             }
         }
 
+        let mut rust_flags = vec![];
+
         // Windows standalone_static distributions require the non-DLL CRT.
         // This requires telling Rust to use the static CRT.
         //
@@ -278,7 +257,7 @@ pub fn build_executable_with_rust_project<'a>(
         .to_embedded_python_context(logger, env, opt_level)
         .context("obtaining embedded python context")?;
     embedded_data
-        .write_files(&artifacts_path)
+        .write_files(artifacts_path)
         .context("writing embedded python context files")?;
 
     let build_env = BuildEnvironment::new(
@@ -286,9 +265,8 @@ pub fn build_executable_with_rust_project<'a>(
         logger,
         exe.target_triple(),
         artifacts_path,
-        exe.target_python_exe_path(),
+        embedded_data.pyo3_config_path(&artifacts_path),
         exe.libpython_link_mode(),
-        embedded_data.linking_info.libpython_filename.as_deref(),
         exe.apple_sdk_info(),
     )
     .context("resolving build environment")?;
@@ -324,14 +302,6 @@ pub fn build_executable_with_rust_project<'a>(
     args.push("--no-default-features");
     let mut features = vec!["build-mode-prebuilt-artifacts"];
 
-    // If we have a real libpython, let cpython crate link against it. Otherwise
-    // leave symbols unresolved, as we'll provide them.
-    features.push(if embedded_data.linking_info.libpython_filename.is_some() {
-        "cpython-link-default"
-    } else {
-        "cpython-link-unresolved-static"
-    });
-
     if exe.requires_jemalloc() {
         features.push("global-allocator-jemalloc");
         features.push("allocator-jemalloc");
@@ -500,9 +470,10 @@ pub fn build_pyembed_artifacts(
     for target in context.targets_to_resolve()? {
         let resolved: ResolvedTarget = context.build_resolved_target(&target)?;
 
-        let cargo_metadata = resolved.output_path.join("cargo_metadata.txt");
-
-        if !cargo_metadata.exists() {
+        // Presence of the generated default python config file implies this is a valid
+        // artifacts directory.
+        let default_python_config = resolved.output_path.join(DEFAULT_PYTHON_CONFIG_FILENAME);
+        if !default_python_config.exists() {
             continue;
         }
 
@@ -520,13 +491,13 @@ pub fn build_pyembed_artifacts(
             ))?;
         }
 
-        // TODO should we normalize paths to pyoxidizer build directory in cargo_metadata.txt
-        // with the new artifacts directory?
-
         return Ok(());
     }
 
-    Err(anyhow!("unable to find generated cargo_metadata.txt; did you specify the correct target to resolve?"))
+    Err(anyhow!(
+        "unable to find generated {}; did you specify the correct target to resolve?",
+        DEFAULT_PYTHON_CONFIG_FILENAME
+    ))
 }
 
 /// Runs packaging/embedding from the context of a Rust build script.
@@ -600,12 +571,11 @@ pub fn run_from_build(
         false,
     )?;
 
-    let cargo_metadata = dest_dir.join("cargo_metadata.txt");
-
-    let content =
-        std::fs::read(&cargo_metadata).context(format!("reading {}", cargo_metadata.display()))?;
-    let content = String::from_utf8(content).context("converting cargo_metadata.txt to string")?;
-    println!("{}", content);
+    let default_python_config_path = dest_dir.join(DEFAULT_PYTHON_CONFIG_FILENAME);
+    println!(
+        "cargo:rustc-env=DEFAULT_PYTHON_CONFIG_RS={}",
+        default_python_config_path.display()
+    );
 
     Ok(())
 }
@@ -643,23 +613,23 @@ fn dependency_current(
 
 /// Determines whether PyOxidizer artifacts are current.
 fn artifacts_current(logger: &slog::Logger, config_path: &Path, artifacts_path: &Path) -> bool {
-    let metadata_path = artifacts_path.join("cargo_metadata.txt");
+    let python_config_path = artifacts_path.join(DEFAULT_PYTHON_CONFIG_FILENAME);
 
-    if !metadata_path.exists() {
+    if !python_config_path.exists() {
         warn!(logger, "no existing PyOxidizer artifacts found");
         return false;
     }
 
     // We assume the mtime of the metadata file is the built time. If we
     // encounter any modified times newer than that file, we're not up to date.
-    let built_time = match metadata_path.metadata() {
+    let built_time = match python_config_path.metadata() {
         Ok(md) => match md.modified() {
             Ok(t) => t,
             Err(_) => {
                 warn!(
                     logger,
                     "error determining mtime of {}",
-                    metadata_path.display()
+                    python_config_path.display()
                 );
                 return false;
             }
@@ -668,30 +638,12 @@ fn artifacts_current(logger: &slog::Logger, config_path: &Path, artifacts_path:
             warn!(
                 logger,
                 "error resolving metadata of {}",
-                metadata_path.display()
+                python_config_path.display()
             );
             return false;
         }
     };
 
-    let metadata_data = match std::fs::read_to_string(&metadata_path) {
-        Ok(data) => data,
-        Err(_) => {
-            warn!(logger, "error reading {}", metadata_path.display());
-            return false;
-        }
-    };
-
-    for line in metadata_data.split('\n') {
-        if line.starts_with("cargo:rerun-if-changed=") {
-            let path = PathBuf::from(&line[23..line.len()]);
-
-            if !dependency_current(logger, &path, built_time) {
-                return false;
-            }
-        }
-    }
-
     let current_exe = std::env::current_exe().expect("unable to determine current exe");
     if !dependency_current(logger, &current_exe, built_time) {
         return false;
