@@ -6,21 +6,26 @@
 
 use {
     crate::{
-        environment::{default_target_triple, Environment, PyOxidizerSource},
+        environment::{canonicalize_path, default_target_triple, Environment, PyOxidizerSource},
         project_building::find_pyoxidizer_config_file_env,
         project_layout::{initialize_project, write_new_pyoxidizer_config_file},
         py_packaging::{
             distribution::{
                 default_distribution_location, resolve_distribution,
-                resolve_python_distribution_archive, DistributionFlavor,
+                resolve_python_distribution_archive, BinaryLibpythonLinkMode, DistributionCache,
+                DistributionFlavor, PythonDistribution,
             },
             standalone_distribution::StandaloneDistribution,
         },
+        python_distributions::PYTHON_DISTRIBUTIONS,
         starlark::eval::EvaluationContextBuilder,
     },
     anyhow::{anyhow, Context, Result},
     python_packaging::{
-        filesystem_scanning::find_python_resources, resource::PythonResource, wheel::WheelArchive,
+        filesystem_scanning::find_python_resources,
+        interpreter::{MemoryAllocatorBackend, PythonInterpreterProfile},
+        resource::PythonResource,
+        wheel::WheelArchive,
     },
     std::{
         collections::HashMap,
@@ -28,7 +33,7 @@ use {
         io::{Cursor, Read},
         path::{Path, PathBuf},
     },
-    tugger_file_manifest::FileData,
+    tugger_file_manifest::{FileData, FileManifest},
     tugger_licensing::LicenseFlavor,
 };
 
@@ -356,26 +361,26 @@ pub fn init_rust_project(
         "A new Rust binary application has been created in {}",
         project_path.display()
     );
-    println!();
-    println!("This application can be built by doing the following:");
-    println!();
-    println!("  $ cd {}", project_path.display());
-    println!("  $ pyoxidizer build");
-    println!("  $ pyoxidizer run");
-    println!();
-    println!("The default configuration is to invoke a Python REPL. You can");
-    println!("edit the various pyoxidizer.*.bzl config files or the main.rs ");
-    println!("file to change behavior. The application will need to be rebuilt ");
-    println!("for configuration changes to take effect.");
-    println!();
-    println!("IMPORTANT: use of `cargo` for direct project building and running");
-    println!("is possible, but likely requires setting environment variables");
-    println!("like PYOXIDIZER_EXE (the path to the `pyoxidizer` that the build.rs");
-    println!("build script should use) and PYTHON_SYS_EXECUTABLE (the path to the");
-    println!("Python interpreter executable used to configure the Rust crates that");
-    println!("link against libpython). Search the documentation for references to");
-    println!("these variables for troubleshooting tips. For best results, use");
-    println!("the aforementioned `pyoxidizer` commands to build Rust projects.");
+    print!(
+        r#"
+This application can be built most easily by doing the following:
+
+  $ cd {project_path}
+  $ pyoxidizer run
+
+Note however that this will bypass all the Rust code in the project
+folder, and build the project as if you had only created a pyoxidizer.bzl
+file. Building from Rust is more involved, and requires multiple steps.
+Please see the "PyOxidizer Rust Projects" section of the manual for more
+information.
+
+The default configuration is to invoke a Python REPL. You can
+edit the various pyoxidizer.*.bzl config files or the main.rs
+file to change behavior. The application will need to be rebuilt
+for configuration changes to take effect.
+"#,
+        project_path = project_path.display()
+    );
 
     Ok(())
 }
@@ -576,3 +581,100 @@ pub fn python_distribution_licenses(path: &str) -> Result<()> {
 
     Ok(())
 }
+
+/// Generate artifacts for embedding Python in a binary.
+pub fn generate_python_embedding_artifacts(
+    env: &Environment,
+    logger: &slog::Logger,
+    target_triple: &str,
+    flavor: &str,
+    python_version: Option<&str>,
+    dest_path: &Path,
+) -> Result<()> {
+    let flavor = DistributionFlavor::try_from(flavor).map_err(|e| anyhow!("{}", e))?;
+
+    std::fs::create_dir_all(dest_path)
+        .with_context(|| format!("creating directory {}", dest_path.display()))?;
+
+    let dest_path = canonicalize_path(dest_path).context("canonicalizing destination directory")?;
+
+    let distribution_record = PYTHON_DISTRIBUTIONS
+        .find_distribution(target_triple, &flavor, python_version)
+        .ok_or_else(|| anyhow!("could not find Python distribution matching requirements"))?;
+
+    let distribution_cache = DistributionCache::new(Some(&env.python_distributions_dir()));
+
+    let dist = distribution_cache
+        .resolve_distribution(logger, &distribution_record.location, None)
+        .context("resolving Python distribution")?;
+
+    let host_dist = distribution_cache
+        .host_distribution(
+            logger,
+            Some(dist.python_major_minor_version().as_str()),
+            None,
+        )
+        .context("resolving host distribution")?;
+
+    let policy = dist
+        .create_packaging_policy()
+        .context("creating packaging policy")?;
+
+    let mut interpreter_config = dist
+        .create_python_interpreter_config()
+        .context("creating Python interpreter config")?;
+
+    interpreter_config.config.profile = PythonInterpreterProfile::Python;
+    interpreter_config.allocator_backend = MemoryAllocatorBackend::Default;
+
+    let mut builder = dist.as_python_executable_builder(
+        logger,
+        default_target_triple(),
+        target_triple,
+        "python",
+        BinaryLibpythonLinkMode::Default,
+        &policy,
+        &interpreter_config,
+        Some(host_dist.clone_trait()),
+    )?;
+
+    builder.set_tcl_files_path(Some("tcl".to_string()));
+
+    builder
+        .add_distribution_resources(None)
+        .context("adding distribution resources")?;
+
+    let embedded_context = builder
+        .to_embedded_python_context(logger, env, "1")
+        .context("resolving embedded context")?;
+
+    embedded_context
+        .write_files(&dest_path)
+        .context("writing embedded artifact files")?;
+
+    embedded_context
+        .extra_files
+        .materialize_files(&dest_path)
+        .context("writing extra files")?;
+
+    // Write out a copy of the standard library.
+    let mut m = FileManifest::default();
+    for resource in find_python_resources(
+        &dist.stdlib_path,
+        dist.cache_tag(),
+        &dist.python_module_suffixes()?,
+        true,
+        false,
+    ) {
+        if let PythonResource::File(file) = resource? {
+            m.add_file_entry(file.path(), file.entry())?;
+        } else {
+            panic!("find_python_resources() should only emit File variant");
+        }
+    }
+
+    m.materialize_files_with_replace(dest_path.join("stdlib"))
+        .context("writing standard library")?;
+
+    Ok(())
+}
