@@ -101,7 +101,7 @@ impl ToString for PyembedPackedResourcesSource {
             Self::MemoryMappedPath(path) => {
                 format!(
                     "pyembed::PackedResourcesSource::MemoryMappedPath({})",
-                    path_to_string(&path)
+                    path_to_string(path)
                 )
             }
         }
@@ -500,8 +500,10 @@ mod tests {
 
     #[test]
     fn test_backslash_in_path() -> Result<()> {
-        let mut config = PyembedPythonInterpreterConfig::default();
-        config.tcl_library = Some(PathBuf::from("c:\\windows"));
+        let config = PyembedPythonInterpreterConfig {
+            tcl_library: Some(PathBuf::from("c:\\windows")),
+            ..Default::default()
+        };
 
         let code = config.to_oxidized_python_interpreter_config_rs()?;
 
@@ -517,7 +519,7 @@ mod tests {
     fn test_build_all_fields() -> Result<()> {
         let env = get_env()?;
         let logger = get_logger()?;
-        let dist = get_default_distribution()?;
+        let dist = get_default_distribution(None)?;
         let policy = dist.create_packaging_policy()?;
 
         let config = PyembedPythonInterpreterConfig {
