@@ -23,7 +23,6 @@ use {
     slog::warn,
     std::{
         collections::HashMap,
-        convert::TryFrom,
         fs,
         fs::{create_dir_all, File},
         io::Read,
@@ -54,6 +53,19 @@ pub enum PythonDistributionLocation {
     Url { url: String, sha256: String },
 }
 
+impl std::fmt::Display for PythonDistributionLocation {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        match self {
+            Self::Local { local_path, sha256 } => {
+                write!(f, "{} (sha256={})", local_path, sha256)
+            }
+            Self::Url { url, sha256 } => {
+                write!(f, "{} (sha256={})", url, sha256)
+            }
+        }
+    }
+}
+
 /// Describes an obtainable Python distribution.
 #[derive(Clone, Debug, PartialEq)]
 pub struct PythonDistributionRecord {
@@ -136,6 +148,9 @@ pub trait PythonDistribution {
     /// Obtain file suffixes for various Python module flavors.
     fn python_module_suffixes(&self) -> Result<PythonModuleSuffixes>;
 
+    /// Python configuration variables.
+    fn python_config_vars(&self) -> &HashMap<String, String>;
+
     /// Obtain Python packages in the standard library that provide tests.
     fn stdlib_test_packages(&self) -> Vec<String>;
 
@@ -355,7 +370,7 @@ pub fn copy_local_distribution(path: &Path, sha256: &str, cache_dir: &Path) -> R
         }
     }
 
-    let source_hash = sha256_path(&path);
+    let source_hash = sha256_path(path);
 
     if source_hash != expected_hash {
         return Err(anyhow!("sha256 of Python distribution does not validate"));
@@ -403,7 +418,7 @@ pub fn resolve_python_distribution_from_location(
     location: &PythonDistributionLocation,
     distributions_dir: &Path,
 ) -> Result<(PathBuf, PathBuf)> {
-    warn!(logger, "resolving Python distribution {:?}", location);
+    warn!(logger, "resolving Python distribution {}", location);
     let path = resolve_python_distribution_archive(location, distributions_dir)?;
     warn!(
         logger,
@@ -546,7 +561,7 @@ impl DistributionCache {
             Ok(dist.clone())
         } else {
             let dist = Arc::new(StandaloneDistribution::from_location(
-                logger, location, &dest_dir,
+                logger, location, dest_dir,
             )?);
 
             lock.replace(dist.clone());
@@ -554,6 +569,24 @@ impl DistributionCache {
             Ok(dist)
         }
     }
+
+    /// Resolve a Python distribution that runs on the current machine.
+    pub fn host_distribution(
+        &self,
+        logger: &slog::Logger,
+        python_major_minor_version: Option<&str>,
+        dest_dir: Option<&Path>,
+    ) -> Result<Arc<StandaloneDistribution>> {
+        let location = default_distribution_location(
+            &DistributionFlavor::Standalone,
+            crate::environment::default_target_triple(),
+            python_major_minor_version,
+        )
+        .context("resolving host distribution location")?;
+
+        self.resolve_distribution(logger, &location, dest_dir)
+            .context("resolving host distribution from location")
+    }
 }
 
 /// Obtain a `PythonDistribution` implementation of a flavor and from a location.
@@ -566,7 +599,7 @@ pub fn resolve_distribution(
     dest_dir: &Path,
 ) -> Result<Box<dyn PythonDistribution>> {
     Ok(Box::new(StandaloneDistribution::from_location(
-        logger, &location, dest_dir,
+        logger, location, dest_dir,
     )?) as Box<dyn PythonDistribution>)
 }
 
