@@ -16,7 +16,6 @@ use {
     starlark_dialect_build_targets::{get_context_value, EnvironmentContext},
     std::{
         collections::HashMap,
-        convert::TryFrom,
         path::{Path, PathBuf},
         sync::Arc,
     },
@@ -154,14 +153,9 @@ impl TypedValue for PyOxidizerEnvironmentContext {
 }
 
 /// Starlark type holding context for PyOxidizer.
+#[derive(Default)]
 pub struct PyOxidizerContext {}
 
-impl Default for PyOxidizerContext {
-    fn default() -> Self {
-        PyOxidizerContext {}
-    }
-}
-
 impl TypedValue for PyOxidizerContext {
     type Holder = Mutable<PyOxidizerContext>;
     const TYPE: &'static str = "PyOxidizer";
